"use strict";

require("core-js/modules/es.symbol.js");

require("core-js/modules/es.symbol.description.js");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.array.map.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.string.split.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = extensionResolver;

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var PLUGIN_NAME = "babel-plugin-module-extension-resolver";
var DEFAULT_EXTENSIONS = [".mjs", ".js", ".es", ".es6", ".ts", ".node", ".json"];
var DEFAULT_MAP = {
  ".ts": ".js",
  ".es": ".js",
  ".es6": ".js",
  ".node": ".js"
};

function extensionResolver(plugin, options) {
  var types = plugin.types;
  var _options$extensions = options.extensions,
      extensions = _options$extensions === void 0 ? DEFAULT_EXTENSIONS : _options$extensions,
      _options$map = options.map,
      map = _options$map === void 0 ? DEFAULT_MAP : _options$map;
  return {
    PLUGIN_NAME,
    visitor: {
      Program: {
        enter(programPath, state) {
          var filename = state.file.opts.filename;
          programPath.traverse({
            CallExpression(declaration) {
              var callee = declaration.get("callee");

              if (!types.isIdentifier(callee) || callee.node.name !== "require") {
                return;
              }

              var args = declaration.get("arguments");

              if (args.length !== 1) {
                return;
              }

              replaceSource(types, args[0], filename, extensions, map);
            },

            ImportDeclaration(declaration) {
              replaceSource(types, declaration.get("source"), filename, extensions, map);
            }

          }, state);
        }

      }
    }
  };
}

function replaceSource(types, source, fileName, extensions, map) {
  if (!types.isStringLiteral(source)) {
    return;
  }

  var sourcePath = source.node.value;

  if (sourcePath[0] !== ".") {
    return;
  }

  var baseDir = _path.default.dirname(fileName);

  var resolvedPath = resolvePath(baseDir, sourcePath, extensions, map);
  var normalizedPath = normalizePath(resolvedPath);
  source.replaceWith(types.stringLiteral(normalizedPath));
}

function resolvePath(baseDir, sourcePath, extensions, map) {
  {
    var resolvedPath = resolvePathCore(baseDir, sourcePath, extensions, map);

    if (resolvedPath !== null) {
      return resolvedPath;
    }
  }

  var absolutePath = _path.default.join(baseDir, sourcePath);

  if (isDirectory(absolutePath)) {
    var _resolvedPath = resolvePathCore(baseDir, _path.default.join(sourcePath, "index"), extensions, map);

    if (_resolvedPath !== null) {
      return _resolvedPath;
    }
  }

  return sourcePath;
}

function resolvePathCore(baseDir, sourcePath, extensions, map) {
  var absolutePath = _path.default.join(baseDir, sourcePath);

  if (isFile(absolutePath)) {
    return sourcePath;
  }

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = extensions[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var extension = _step.value;
      var resolvedPath = `${absolutePath}${extension}`;

      if (!isFile(resolvedPath)) {
        continue;
      }

      if (map.hasOwnProperty(extension)) {
        return _path.default.relative(baseDir, `${absolutePath}${map[extension]}`);
      } else {
        return _path.default.relative(baseDir, resolvedPath);
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return null;
}

function normalizePath(originalPath) {
  if (_path.default.sep === "\\") {
    originalPath = originalPath.split(_path.default.sep).join("/");
  }

  if (originalPath[0] !== ".") {
    originalPath = `./${originalPath}`;
  }

  return originalPath;
}

function isFile(pathName) {
  try {
    return _fs.default.statSync(pathName).isFile();
  } catch (err) {
    return false;
  }
}

function isDirectory(pathName) {
  try {
    return _fs.default.statSync(pathName).isDirectory();
  } catch (err) {
    return false;
  }
}
//# sourceMappingURL=index.js.map