import "core-js/modules/es.array.iterator.js";
import fs from "fs";
import path from "path";
const PLUGIN_NAME = "babel-plugin-module-extension-resolver";
const DEFAULT_EXTENSIONS = [".mjs", ".js", ".es", ".es6", ".ts", ".node", ".json"];
const DEFAULT_MAP = {
  ".ts": ".js",
  ".es": ".js",
  ".es6": ".js",
  ".node": ".js"
};
export default function extensionResolver(plugin, options) {
  const {
    types
  } = plugin;
  const {
    extensions = DEFAULT_EXTENSIONS,
    map = DEFAULT_MAP
  } = options;
  return {
    PLUGIN_NAME,
    visitor: {
      Program: {
        enter(programPath, state) {
          const {
            file: {
              opts: {
                filename
              }
            }
          } = state;
          programPath.traverse({
            CallExpression(declaration) {
              const callee = declaration.get("callee");

              if (!types.isIdentifier(callee) || callee.node.name !== "require") {
                return;
              }

              const args = declaration.get("arguments");

              if (args.length !== 1) {
                return;
              }

              replaceSource(types, args[0], filename, extensions, map);
            },

            ImportDeclaration(declaration) {
              replaceSource(types, declaration.get("source"), filename, extensions, map);
            }

          }, state);
        }

      }
    }
  };
}

function replaceSource(types, source, fileName, extensions, map) {
  if (!types.isStringLiteral(source)) {
    return;
  }

  const sourcePath = source.node.value;

  if (sourcePath[0] !== ".") {
    return;
  }

  const baseDir = path.dirname(fileName);
  const resolvedPath = resolvePath(baseDir, sourcePath, extensions, map);
  const normalizedPath = normalizePath(resolvedPath);
  source.replaceWith(types.stringLiteral(normalizedPath));
}

function resolvePath(baseDir, sourcePath, extensions, map) {
  {
    const resolvedPath = resolvePathCore(baseDir, sourcePath, extensions, map);

    if (resolvedPath !== null) {
      return resolvedPath;
    }
  }
  const absolutePath = path.join(baseDir, sourcePath);

  if (isDirectory(absolutePath)) {
    const resolvedPath = resolvePathCore(baseDir, path.join(sourcePath, "index"), extensions, map);

    if (resolvedPath !== null) {
      return resolvedPath;
    }
  }

  return sourcePath;
}

function resolvePathCore(baseDir, sourcePath, extensions, map) {
  const absolutePath = path.join(baseDir, sourcePath);

  if (isFile(absolutePath)) {
    return sourcePath;
  }

  for (const extension of extensions) {
    const resolvedPath = `${absolutePath}${extension}`;

    if (!isFile(resolvedPath)) {
      continue;
    }

    if (map.hasOwnProperty(extension)) {
      return path.relative(baseDir, `${absolutePath}${map[extension]}`);
    } else {
      return path.relative(baseDir, resolvedPath);
    }
  }

  return null;
}

function normalizePath(originalPath) {
  if (path.sep === "\\") {
    originalPath = originalPath.split(path.sep).join("/");
  }

  if (originalPath[0] !== ".") {
    originalPath = `./${originalPath}`;
  }

  return originalPath;
}

function isFile(pathName) {
  try {
    return fs.statSync(pathName).isFile();
  } catch (err) {
    return false;
  }
}

function isDirectory(pathName) {
  try {
    return fs.statSync(pathName).isDirectory();
  } catch (err) {
    return false;
  }
}
//# sourceMappingURL=index.mjs.map